/*
 * =====================================================================================
 *
 *       Filename:  Attenuator.h
 *
 *    Description:  Attenuator
 *
 *        Version:  1.0
 *        Created:  13.12.2019 13:54:49
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
 *   Organization:  Budker Insitute of Nuclear Physics
 *
 * =====================================================================================
 */
#ifndef IBN_CAMAC_DEVICE_ATTENUATOR_H
#define IBN_CAMAC_DEVICE_ATTENUATOR_H
#include "Module.h"
#include "../device/Attenuator.h"

namespace camac 
{

  template<int MAX_ATTENUATION, int STEP>
  class Attenuator_A0608 : public device::Attenuator,  protected  Module
  {
	  public:
		  double GetMaxAttenuation(void) const final { return MAX_ATTENUATION; } 
		  double GetStep(void) const  final { return STEP;}
		  const device::Info & GetInfo(void) const final { return Module::GetInfo(); }
		  Attenuator_A0608(const camac::Address adr, std::string name) : Module(adr,"Attenuator", "CAMAC/A0608", name) 
	  	  { 
				info.version = std::to_string(MAX_ATTENUATION);
		  }
		  virtual ~Attenuator_A0608(void) {}

		  void SetAttenuation(double att) final
		  {
			  AF(0,16,std::bitset<16>(static_cast<uint16_t>(att/STEP)));
		  }

		  double GetAttenuation(void) const final 
		  {
			  return static_cast<double>(( AF<16>(0,0) & std::bitset<16>(0xf)).to_ulong())*STEP;
		  }

  };

  typedef Attenuator_A0608<15,1>   Attenuator_A0608_15;
  typedef Attenuator_A0608<45,3>   Attenuator_A0608_45;

}
#endif
