#ifndef IBN_DELAYLINE_H
#define IBN_DELAYLINE_H
#include "Module.h"

#include <fmt/format.h>
#include <fmt/chrono.h>

#include <chrono>

#include "../device/DelayLine.h"

namespace camac	
{
	class DelayLine_L0606 : public device::DelayLine, protected Module 
	{
		/*
		   Разработчик Бехтенев Евгений Алексеевич, ИЯФ, тел 48-00
		   Developer   Bekhtenev Evgeny Alexeevich, BINP, ph. +7 383-329-48-00
		   */
		const delay_t Step;//минимальный шаг
		const delay_t MaxDelay; //256*5;
		const delay_t MinDelay = std::chrono::nanoseconds(0);
		public:

		virtual const device::Info & GetInfo(void) const final { return Module::GetInfo(); }

		DelayLine_L0606(std::string nm, const Address adr, delay_t step) : 
			Module({adr.interface, adr.crate, adr.position, 0} , "DelayLine","CAMAC/L0606", nm),
			Step(step),
			MaxDelay(Step*256)
		{
		}

		DelayLine_L0606(std::string nm, const Address adr, delay_t step, delay_t delay) : 
			Module({adr.interface, adr.crate, adr.position, 0} , "DelayLine","CAMAC/L0606", nm),
			Step(step),
			MaxDelay(Step*256)
		{
			SetDelay(delay);
		}

		~DelayLine_L0606(void){}

		virtual void SetDelay(const delay_t & delay) final
		{
			if(delay < MinDelay || delay > MaxDelay) 
			{
				std::string what = Module::GetInfo().Type() + ": Bad delay time for module L0606(Delay line) ";
				what+= fmt::format("delay {0} <  MinDelay {1} OR delay {0} > MaxDelay {2}", delay, MinDelay, MaxDelay);
				throw std::out_of_range(what);
			}
			uint32_t code = static_cast<uint32_t>((delay - MinDelay)/Step);
			//std::cout << "SetDelay: code = " << code << std::endl;
			AF<16>(GetChannel(),16, code);
		}

		delay_t GetDelay(void) const final
		{
			auto data = AF<16>(GetChannel(),0);
			return (255-data.to_ulong())*Step + MinDelay;
		}
	};//end of class LO606 new 


}//end of namespace camac
#endif // IBN_DELAY_LINE_H
