#ifndef IBN_DEVICE_DEPOLARIZER_H
#define IBN_DEVICE_DEPOLARIZER_H
#include "Device.h"
#include <numeric>
#include <chrono>
namespace device 
{
	class Depolarizer : public  Device
	{
		public:
			virtual ~Depolarizer(void){};
			virtual void    start_scan() = 0;
			virtual void     stop_scan() = 0;
			virtual void    pause_scan() = 0;
			virtual void continue_scan() = 0;
			virtual bool       is_scan() const = 0;
			virtual bool         is_on() const = 0;

			virtual void           off() = 0 ;     //turn off the generator
			virtual void            on() = 0;     //turn on the generator
			virtual void            on(double f) = 0; //turn on the generator with spec frequency f in Hz

			virtual void set_attenuation(double a) = 0; //set attenuation
														//virtual void set_attenuation(void) = 0 ;    //set default attenuation

			virtual double get_attenuation(void) const  = 0;

			virtual void set_speed(double) = 0; 
			virtual void set_step(double)  = 0;
			virtual void set_initial(double f) = 0;
			virtual void set_final(double f) = 0;
			virtual void set_frequency(double f) = 0;
			virtual void set_width(double width) = 0;
			virtual void set_delay(double delay) = 0;

			virtual void set_harmonic_number(int n) = 0;
			virtual void set_revolution(double ) = 0;

			virtual double get_frequency(void) const  = 0;
			virtual double get_initial(void) const = 0; 
			virtual double get_final(void) const = 0; 
			virtual double get_speed(void) const = 0;
			virtual double get_step(void) const = 0;
			virtual double get_width(void) const = 0;
			virtual double get_delay(void) const = 0;

			virtual int get_harmonic_number() const = 0;
			virtual double get_revolution(void) const = 0;

			struct State
			{
				static constexpr double MeV = 1.0;
				static constexpr double keV = 1e-3*MeV;
				static constexpr double eV  = 1e-6*MeV;
				static constexpr double Hz =  1.0;
				static constexpr double kHz = 1e3*Hz;
				static constexpr double MHz = 1e6*Hz;
				static constexpr double mHz = 1-3*Hz;
				//предыдущее значение
				//static constexpr double RDM = 440.6484586602595*MeV; // me * mu0/mu'
				//PDG-2024 (CODATA-2021)
				//me = 0.510'998'950'00 +- 0.000'000'000'15  ( погрешность в относительных единицах 0.294e-9 )
				//PDG-2024 average of FAN (2023) ,HANNEK(2008), VANDYCK(1987)
				// mu_e/mu_0 = (g-2_/2 =  ( 1159.652'180'62 +- 0.000'000'12 )*10^-6 ( погрешность в относительных единицах 0.103e-9 )
				// На основе этого вычислен коэффициент me*mu_0/mu, который от предыдущего значения отличается на 7.8e-9 (в относительных единицах)
				static constexpr double RDM = 440.648'462'134 *MeV; // me * mu0/mu'
																	//погрешность       RDM_error = 0.000'000'137 МэВ или 0.3e-9 в относительных единицах

				std::chrono::system_clock::time_point timestamp;
				bool isscan;
				bool ison;
				int harmonic_number;
				double frequency;
				double speed;
				double step;
				double width;
				double attenuation;

				State() = default;
				State(std::chrono::system_clock::time_point tp,
						bool scan,
						bool on,
						int hn,
						double f,
						double spd,
						double stp,
						double wdh,
						double at)  :
					timestamp(tp), isscan(scan), ison(on), harmonic_number(hn), frequency(f), speed(spd), step(stp), width(wdh), attenuation(at)
				{
				}





				auto energy(double f0 = 818924.485 ) const -> double
				{
					return std::abs(frequency) < std::numeric_limits<double>::epsilon() ? 0 :(harmonic_number + frequency/f0)*RDM;
				}

				auto energy_speed(double f0 = 818924.485 ) const -> double
				{
					return  speed/f0*RDM;
				}

				auto energy_step(double f0 = 818924.485 ) const -> double 
				{
					return step/f0*RDM;
				}

				auto energy_width(double f0 = 818924.485 ) const -> double  
				{
					return width/f0*RDM;
				}

			};

	};


}
#endif
