#ifndef IBN_DEVICE_FAKE_DEPOLARIZER_H
#define IBN_DEVICE_FAKE_DEPOLARIZER_H
#include "Depolarizer.h"

#include <chrono>
#include <thread>

namespace device 
{
	class FakeDepolarizer : public Depolarizer
	{
		std::thread thread;
		device::Info info;
		public:
		const device::Info & GetInro() const final { return info; }

		FakeDepolarizer(std::string name)
		{
			info.name = name;
			info.address = "";
			info.type = "Depolarizer";
			info.model = "FAKE";
			isscan = false;
			ison = false;
		}

		virtual ~FakeDepolarizer(void) {}

		std::string host;
		int port;

		bool isscan;
		bool ison;
		double attenuation=45;
		double speed=1;
		double step=1;
		double initial=500000;
		double final=600000;
		double frequency=0;
		double harmonic_number=4;
		const double revolution_frequency=818924.485;
		double local_ifreq;
		double ifreq;

		std::chrono::nanoseconds step_time; //time for scan step, ns
		std::chrono::steady_clock::time_point begin_time;
		std::chrono::steady_clock::time_point local_begin_time;

		void start_scan()     final;

		void stop_scan()      { isscan = false; thread.join(); } 
		void pause_scan()     { } 
		void continue_scan()  { } 
		void on()             { } 
		void off()            { } 

		bool is_scan(void) const  {return isscan; };
		bool is_on() const {return ison; }

		void set_attenuation(double a)     { attenuation = a;}
		void set_speed(double a)           { speed = a;}
		void set_step(double a)            { step = a;}
		void set_initial(double a)         { initial = a;}
		void set_final(double a)           { final = a;}
		void set_frequency(double a)       { frequency = a;}
		void set_harmonic_number(double a) { harmonic_number = a;}

		double get_attenuation(void)     { return  attenuation;}
		double get_speed(void)           { return speed;}
		double get_step(void)            { return step;}
		double get_initial(void)         { return initial;}
		double get_final(void)           { return final;}
		double get_frequency(void)       { return frequency;}
		double get_harmonic_number(void) { return harmonic_number; }
		double get_revolution_frequency(void) { return revolution_frequency; }
	};

	void Depolarizer::start_scan(void)
	{
		if (thread.joinable() ) return;
		using namespace std::chrono;
		using namespace std::chrono_literals;
		isscan = true;
		local_ifreq = ifreq;
		begin_time = steady_clock::now();
		local_begin_time = begin_time;
		begin_time = steady_clock::now();
		thread = std::thread 
			( 
			 [this](void) {
			 frequency=local_ifreq;
			 while(isscan) {
			 auto t = steady_clock::now();
			 duration<double> elapsed_time = t-local_begin_time;
			 frequency = local_ifreq + speed*elapsed_time.count();
			 while( isscan && std::chrono::steady_clock::now()<t+step_time) {
			 auto sleep_time = 50ms < step_time ? 50ms : step_time;
			 std::this_thread::sleep_for(sleep_time);
			 }
			 }
			 }
			);
	}
}
#endif
