#ifndef IBN_DEVICE_FAKE_POWER_SUPPLY_H
#define IBN_DEVICE_FAKE_POWER_SUPPLY_H

#include "PowerSupply.h"

namespace device 
{
	class FakePowerSupply : public device::Info, public device::PowerSupply 
	{
			const double StepVoltage;
			const double MaxVoltage;
			double V=0;
			bool ison=false;
			bool is_voltage_overload = false;
			bool is_current_overload = false;
		public:
			virtual const Info & GetInfo(void) const  final 
			{
				return static_cast<const Info&>(*this);
			}

			virtual ~FakePowerSupply() {};

			FakePowerSupply(std::string nm="", std::string a="", double step_voltage=1.0, double max_voltage=4000.0) 
				: StepVoltage{step_voltage}, 
				MaxVoltage{max_voltage}
			{
				type = "PowerSupply";
				model = "FAKE";
				address = a;
				name = nm;
			}

			void On(void) final { ison = true;}
			void Off(void) final { ison = false; }
			void Reset(void) final {}
			void SetVoltage(double v) final { V = v;} 
			double GetVoltage(void ) const final { return V; }
			double GetMaxVoltage(void) const final { return MaxVoltage; }
			double GetStepVoltage(void) const final { return StepVoltage; }
			bool   IsOverload(void) const  final { return is_voltage_overload || is_current_overload; }
			bool   IsVoltageOverload(void) const  final { return is_voltage_overload; }
			bool   IsCurrentOverload(void) const  final { return is_current_overload; }
			bool   IsOn(void) const final { return ison; }

			//specific methods
			void SetVoltageOverload(bool b) { is_voltage_overload = b; }
			void SetCurrentOverload(bool b) { is_current_overload = b; }
	};
}

#endif



