#ifndef IBN_DEPOLARIZER_PRINT_H
#define IBN_DEPOLARIZER_PRINT_H

#include "Depolarizer.h"
#include <fmt/format.h>
#include <fmt/ostream.h>



template <> 
struct fmt::formatter<device::Depolarizer::State> {
	char presentation = ' ';
	constexpr auto parse(format_parse_context & ctx) -> decltype(ctx.begin() ) 
	{
		return ctx.begin();
	}

	template <typename FormatContext>
	auto format (const device::Depolarizer::State & d, FormatContext & ctx) const -> decltype (ctx.out()) 
	{
		return fmt::format_to(ctx.out(), 
				    "{:15.3f} Hz {:15.6f} MeV {:>6.0f} dB {:>10.3f} Hz/s {:>10.3f} keV/s {:>10.3f} {:>10.3}",
					d.frequency, 
					d.energy(), 
					d.attenuation, 
					d.speed, 
					d.energy_speed()*1e3, 
					d.energy_step()*1e3,
					d.energy_width()*1e3
					);
	}

};


template <> 
struct fmt::formatter<device::Info> {
	char presentation = ' ';
	constexpr auto parse(format_parse_context & ctx) -> decltype(ctx.begin() ) 
	{
		return ctx.begin();
	}

	template <typename FormatContext>
	auto format (const device::Info & info, FormatContext & ctx) const -> decltype (ctx.out()) 
	{
		return fmt::format_to(ctx.out(), "{type}:{name}:{model}/{version}:SN:{SN}:adr={address}", 
				fmt::arg("type", info.Type()), 
				fmt::arg("name", info.Name()), 
				fmt::arg("model",info.Model()),
				fmt::arg("version", info.Version()),
				fmt::arg("address", info.Address()), 
				fmt::arg("SN", info.SerialNumber())
				);
	}

};

#endif
