/*
 * =====================================================================================
 *
 *       Filename:  Message.h
 *
 *    Description:  Some helper function to send or receive message
 *
 *        Version:  1.0
 *        Created:  12.10.2017 12:29:23
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ivan B. Nikolaev (ekherit), I.B.Nikolaev@inp.nsk.su
 *   Organization:  Budker Insitute of Nuclear Physics
 *
 * =====================================================================================
 */
#ifndef IBN_DEPOLARIZER_MESSAGE_H
#define IBN_DEPOLARIZER_MESSAGE_H

#include <string>
#include <vector>
#include <chrono>
#include <boost/asio.hpp>
#include <iostream>

using  namespace boost::asio;

namespace depolarizer_message_namespace {
	template<typename Message>
		inline Message receive_message(std::shared_ptr<ip::tcp::socket>  socket) {
			Message msg;
			//msg.Clear();
			std::vector<char> v;
			const size_t max_buf_size = (1<<16);
			v.reserve(max_buf_size);
			//boost::system::error_code ec;
			//read the size of the message
			uint64_t size;
			size_t nbytes = boost::asio::read(*socket, buffer(&size,sizeof(size)));
			if( size > max_buf_size ) {
				std::string s =  "ERROR: size greater then max_buf_size = " +  std::to_string(max_buf_size);
				throw std::runtime_error(s);
			}
			v.resize(size);
			nbytes = boost::asio::read(*socket, buffer(v));
			if ( nbytes != size) {
				std::cerr << "ERROR: wrong number of recived bytes" << std::endl;
			}
			std::string s(&v[0],v.size());
			msg.ParseFromString(s);
			return msg;
		}

	template<typename Message>
		inline void send_message(std::shared_ptr<ip::tcp::socket>  socket, const Message & msg) {
			boost::system::error_code ec;
			Message & m = const_cast<Message & >(msg);
			m.set_timestamp(std::chrono::system_clock::now().time_since_epoch().count());
			std::string s;
			msg.SerializeToString(&s);
			uint64_t size = msg.ByteSizeLong();
			boost::asio::write(*socket, boost::asio::buffer(&size, sizeof(size)));
			boost::asio::write(*socket,boost::asio::buffer(s.c_str(), s.size()));
		}
}
#endif
